@@ -1,5 +1,5 @@
 package AnyEvent::BitTorrent;
-{ $AnyEvent::BitTorrent::VERSION = 'v0.2.3' }
+{ $AnyEvent::BitTorrent::VERSION = 'v0.2.5' }
 use AnyEvent;
 use AnyEvent::Handle;
 use AnyEvent::Socket;
@@ -53,7 +53,6 @@ has port => (is      => 'ro',
 has socket => (is        => 'ro',
                isa       => Ref,
                init_arg  => undef,
-               required  => 1,
                predicate => '_has_socket',
                builder   => '_build_socket'
 );
@@ -94,8 +93,11 @@ has path => (is       => 'ro',
              isa      => $FILE,
              required => 1
 );
-has reserved => (is  => 'lazy',
-                 isa => $RESERVED);
+has reserved => (is         => 'ro',
+                 lazy_build => 1,
+                 builder    => '_build_reserved',
+                 isa        => $RESERVED
+);
 
 sub _build_reserved {
     my $reserved = "\0" x 8;
@@ -105,31 +107,33 @@ sub _build_reserved {
     AE::log debug => '_build_reserved() => ' . $reserved;
     $reserved;
 }
-has peerid => (
-    is       => 'ro',
-    isa      => $PEERID,
-    init_arg => undef,
-    required => 1,
-    default  => sub {
-        pack(
-            'a20',
-            (sprintf(
-                 '-AB%01d%01d%01d%1s-%7s%-5s',
-                 ($AnyEvent::BitTorrent::VERSION =~ m[^v(\d+)\.(\d+)\.(\d+)]),
-                 ($AnyEvent::BitTorrent::VERSION =~ m[[^\d\.^v]] ? 'U' : 'S'),
-                 (join '',
-                  map {
-                      ['A' .. 'Z', 'a' .. 'z', 0 .. 9, qw[- . _ ~]]
-                      ->[rand(66)]
-                  } 1 .. 7
-                 ),
-                 [qw[KaiLi April Aaron]]->[rand 3]
-             )
-            )
-        );
-    }
+has peerid => (is       => 'ro',
+               isa      => $PEERID,
+               init_arg => undef,
+               required => 1,
+               builder  => '_build_peerid'
 );
-has bitfield => (is       => 'lazy',
+
+sub _build_peerid {
+    return pack(
+        'a20',
+        (sprintf(
+             '-AB%01d%01d%01d%1s-%7s%-5s',
+             ($AnyEvent::BitTorrent::VERSION =~ m[^v(\d+)\.(\d+)\.(\d+)]),
+             ($AnyEvent::BitTorrent::VERSION =~ m[[^\d\.^v]] ? 'U' : 'S'),
+             (join '',
+              map {
+                  ['A' .. 'Z', 'a' .. 'z', 0 .. 9, qw[- . _ ~]]->[rand(66)]
+              } 1 .. 7
+             ),
+             [qw[KaiLi April Aaron]]->[rand 3]
+         )
+        )
+    );
+}
+has bitfield => (is       => 'ro',
+                 lazy     => 1,
+                 builder  => '_build_bitfield',
                  isa      => Str,
                  init_arg => undef,
 );
@@ -168,15 +172,20 @@ has $_ => (is      => 'ro',
            default => sub {0},
            writer  => '_set_' . $_
 ) for qw[uploaded downloaded];
-has infohash => (is       => 'lazy',
+has infohash => (is       => 'ro',
+                 lazy     => 1,
+                 builder  => '_build_infohash',
                  isa      => $INFOHASH,
-                 init_arg => undef,
-                 default  => sub { sha1(bencode(shift->metadata->{info})) }
-);
-has metadata => (is       => 'lazy',
-                 isa      => HashRef,
                  init_arg => undef
 );
+sub _build_infohash { sha1(bencode(shift->metadata->{info})) }
+has metadata => (is         => 'ro',
+                 lazy_build => 1,
+                 builder    => '_build_metadata',
+                 lazy       => 1,
+                 isa        => HashRef,
+                 init_arg   => undef
+);
 
 sub _build_metadata {
     my $s = shift;
@@ -201,7 +210,8 @@ sub piece_count {
     int($count) + (($count == int $count) ? 1 : 0);
 }
 has basedir => (
-    is       => 'lazy',
+    is       => 'ro',
+    lazy     => 1,
     isa      => Str,
     required => 1,
     default  => sub { File::Spec->rel2abs(File::Spec->curdir) },
@@ -211,7 +221,9 @@ has basedir => (
         $s->_clear_files;    # So they can be rebuilt with the new basedir
     }
 );
-has files => (is       => 'lazy',
+has files => (is       => 'ro',
+              lazy     => 1,
+              builder  => '_build_files',
               isa      => ArrayRef [HashRef],
               init_arg => undef,
               clearer  => '_clear_files'
@@ -245,7 +257,12 @@ sub _build_files {
           }
         ];
 }
-has size => (is => 'lazy', isa => Int, init_arg => undef);
+has size => (is       => 'ro',
+             lazy     => 1,
+             builder  => '_build_size',
+             isa      => Int,
+             init_arg => undef
+);
 
 sub _build_size {
     my $s   = shift;
@@ -510,7 +527,8 @@ sub hashcheck (;@) {
             : $s->_trigger_hash_fail($index);
     }
 }
-has peers => (is      => 'lazy',
+has peers => (is      => 'ro',
+              lazy    => 1,
               isa     => HashRef,
               clearer => '_clear_peers',
               builder => '_build_peers'
@@ -558,61 +576,61 @@ sub _del_peer {
     $h->destroy;
 }
 my $shuffle;
-has trackers => (
-    is       => 'lazy',
-    isa      => ArrayRef [HashRef],
-    required => 1,
-    init_arg => undef,
-    default  => sub {
-        my $s = shift;
-        $shuffle //= sub {
-            my $deck = shift;    # $deck is a reference to an array
-            return unless @$deck;    # must not be empty!
-            my $i = @$deck;
-            while (--$i) {
-                my $j = int rand($i + 1);
-                @$deck[$i, $j] = @$deck[$j, $i];
-            }
-        };
-        my $trackers = [
-            map {
-                {urls       => $_,
-                 complete   => 0,
-                 incomplete => 0,
-                 peers      => '',
-                 peers6     => '',
-                 announcer  => undef,
-                 ticker     => AE::timer(
-                     1,
-                     15 * 60,
-                     sub {
-                         return if $s->state eq 'stopped';
-                         $s->announce('started');
-                     }
-                 ),
-                 failures => 0
-                }
-                } defined $s->metadata->{announce}
-            ? [$s->metadata->{announce}]
-            : (),
-            defined $s->metadata->{'announce-list'}
-            ? @{$s->metadata->{'announce-list'}}
-            : ()
-        ];
-        AE::log trace => sub {
-            require Data::Dump;
-            '$trackers before shuffle => ' . Data::Dump::dump($trackers);
-        };
-        $shuffle->($trackers);
-        $shuffle->($_->{urls}) for @$trackers;
-        AE::log trace => sub {
-            require Data::Dump;
-            '$trackers after shuffle => ' . Data::Dump::dump($trackers);
-        };
-        $trackers;
-    }
+has trackers => (is       => 'ro',
+                 lazy     => 1,
+                 builder  => '_build_trackers',
+                 isa      => ArrayRef [HashRef],
+                 init_arg => undef
 );
 
+sub _build_trackers {
+    my $s = shift;
+    $shuffle //= sub {
+        my $deck = shift;    # $deck is a reference to an array
+        return unless @$deck;    # must not be empty!
+        my $i = @$deck;
+        while (--$i) {
+            my $j = int rand($i + 1);
+            @$deck[$i, $j] = @$deck[$j, $i];
+        }
+    };
+    my $trackers = [
+        map {
+            {urls       => $_,
+             complete   => 0,
+             incomplete => 0,
+             peers      => '',
+             peers6     => '',
+             announcer  => undef,
+             ticker     => AE::timer(
+                 1,
+                 15 * 60,
+                 sub {
+                     return if $s->state eq 'stopped';
+                     $s->announce('started');
+                 }
+             ),
+             failures => 0
+            }
+            } defined $s->metadata->{announce} ? [$s->metadata->{announce}]
+        : (),
+        defined $s->metadata->{'announce-list'}
+        ? @{$s->metadata->{'announce-list'}}
+        : ()
+    ];
+    AE::log trace => sub {
+        require Data::Dump;
+        '$trackers before shuffle => ' . Data::Dump::dump($trackers);
+    };
+    $shuffle->($trackers);
+    $shuffle->($_->{urls}) for @$trackers;
+    AE::log trace => sub {
+        require Data::Dump;
+        '$trackers after shuffle => ' . Data::Dump::dump($trackers);
+    };
+    $trackers;
+}
+
 sub announce {
     my ($s, $e) = @_;
     return if $a++ > 10;    # Retry attempts
@@ -765,7 +783,8 @@ has _fill_requests_timer => (
         );
     }
 );
-has _peer_timer => (is       => 'lazy',
+has _peer_timer => (is       => 'ro',
+                    lazy     => 1,
                     isa      => Ref,
                     init_arg => undef,
                     clearer  => '_clear_peer_timer',
@@ -1119,7 +1138,8 @@ sub _consider_peer {    # Figure out whether or not we find a peer interesting
         }
     }
 }
-has working_pieces => (is       => 'lazy',
+has working_pieces => (is       => 'ro',
+                       lazy     => 1,
                        isa      => HashRef,
                        init_arg => undef,
                        default  => sub { {} }
